local util = require ("util")
local spidertron_animations = require("spidertron-animations")
local function make_spider_leg(number, speed)
  return {
    type = "spider-leg",
    name = "spidertron-leg-" .. number,
    localised_name = {"entity-name.spidertron-leg"},
    collision_box = {{-0.01, -0.01}, {0.01, 0.01}},
    selection_box = {{-0.5, -0.5}, {0.5, 0.5}},
    icon = "__base__/graphics/icons/spidertron.png",
    icon_size = 64, icon_mipmaps = 4,
    walking_sound_volume_modifier = 0.6,
    working_sound = 
    {
      match_progress_to_activity = true,
      sound = sounds.spidertron_leg,
      audible_distance_modifier = 0.5,
    },
    part_length = 3.5,
    initial_movement_speed = 0.06 * speed,
    movement_acceleration = 0.03 * speed,
    max_health = 100,
    movement_based_position_selection_distance = 4,
    selectable_in_game = false,
    graphics_set = spidertron_animations.legs[number],
  }
end

local spider_mx = { 15,  23, 25, 15}
local spider_my = {-22, -10,  4, 17}
local spider_gx = { 2.25, 3, 3, 2.25}
local spider_gy = {-2.5, -1, 1, 2.5}
local spider_legs_per_segment = 4
local spider_segment_mount_offset = 10
local spider_segment_ground_offset = 30
local function make_spider_leg_engine(id, number, side)
	local seg = math.floor(number / spider_legs_per_segment)
	local leg = (number % spider_legs_per_segment) + 1
	local myoff = seg * spider_segment_mount_offset
	local gyoff = seg * spider_segment_ground_offset
	local blocking_legs = {}
	if leg > 1 then
		blocking_legs[#blocking_legs+1] = leg - 1
	end
	if leg < spider_legs_per_segment then
		blocking_legs[#blocking_legs+1] = leg + 1
	end
	return {
		leg = 'spidertron-leg-'..id,
		mount_position = util.by_pixel(spider_mx[leg] * side, spider_my[leg] + myoff),
		ground_position = {spider_gx[leg] * side, spider_gy[leg] + gyoff},
		blocking_legs = blocking_legs,
		leg_hit_the_ground_trigger = get_leg_hit_the_ground_trigger()
	}
end

local function make_resistance(name, dec, per, mult)
	return {type = name, decrease = dec * mult, percent = per * mult}
end

local function make_spider(options)
	-- local options = {
	-- 	name = str,
	-- 	health = 0-1,
	-- 	have_rocket = true,
	-- 	inventory_size = 0-80,
	--  explore_radius = 0-5,
	--  have_eff = true,
	--  have_fusion = true,
	--  leg = ''
	-- }
	local spider = {
    type = "spider-vehicle",
    name = "spidertron"..options.name,
    collision_box = {{-1, -1}, {1, 1}},
    selection_box = {{-1, -1}, {1, 1}},
    icon = "__base__/graphics/icons/spidertron.png",
    mined_sound = {filename = "__core__/sound/deconstruct-large.ogg",volume = 0.8},
    open_sound = { filename = "__base__/sound/spidertron/spidertron-door-open.ogg", volume= 0.35 },
    close_sound = { filename = "__base__/sound/spidertron/spidertron-door-close.ogg", volume = 0.4 },
    sound_minimum_speed = 0.1,
    sound_scaling_ratio = 0.6,
    working_sound =
    {
      sound =
      {
        filename = "__base__/sound/spidertron/spidertron-vox.ogg",
        volume = 0.35
      },
      activate_sound =
      {
        filename = "__base__/sound/spidertron/spidertron-activate.ogg",
        volume = 0.5
      },
      deactivate_sound =
      {
        filename = "__base__/sound/spidertron/spidertron-deactivate.ogg",
        volume = 0.5
      },
      match_speed_to_activity = true,
    },
    icon_size = 64, icon_mipmaps = 4,
    weight = 1,
    braking_force = 1,
    friction_force = 1,
    flags = {"placeable-neutral", "player-creation", "placeable-off-grid"},
    collision_mask = {},
    minable = {mining_time = 1, result = "spidertron"},
    max_health = 3000 * options.health,
    resistances =
    {
      make_resistance("fire", 15, 60, options.health),
      make_resistance("physical", 15, 60, options.health),
      make_resistance("impact", 50, 80, options.health),
      make_resistance("explosion", 20, 75, options.health),
      make_resistance("acid", 0, 70, options.health),
      make_resistance("laser", 0, 70, options.health),
      make_resistance("electric", 0, 70, options.health),
    },
    minimap_representation =
    {
      filename = "__base__/graphics/entity/spidertron/spidertron-map.png",
      flags = {"icon"},
      size = {128, 128},
      scale = 0.5
    },
    corpse = "spidertron-remnants",
    dying_explosion = "spidertron-explosion",
    energy_per_hit_point = 1,
    inventory_size = options.inventory_size,
    height = 1.5,
    torso_rotation_speed = 0.005,
    chunk_exploration_radius = options.explore_radius,
    selection_priority = 51,
    graphics_set = spidertron_animations.torso,
		spider_engine = {
			legs = {
				make_spider_leg_engine(options.leg..'1', 1, 1),
				make_spider_leg_engine(options.leg..'2', 2, 1),
				make_spider_leg_engine(options.leg..'3', 3, 1),
				make_spider_leg_engine(options.leg..'4', 4, 1),
				make_spider_leg_engine(options.leg..'5', 1, -1),
				make_spider_leg_engine(options.leg..'6', 2, -1),
				make_spider_leg_engine(options.leg..'7', 3, -1),
				make_spider_leg_engine(options.leg..'8', 4, -1),
			},
			military_target = "spidertron-military-target",
		}
	}
	if options.have_rocket then
		spider.guns = { "spidertron-rocket-launcher-1", "spidertron-rocket-launcher-2", "spidertron-rocket-launcher-3", "spidertron-rocket-launcher-4" }
    spider.automatic_weapon_cycling = true
    spider.chain_shooting_cooldown_modifier = 0.5
	else
		spider.guns = { "pistol", "pistol", "pistol", "pistol", "pistol", "pistol", "pistol", "pistol"}
    spider.automatic_weapon_cycling = true
    spider.chain_shooting_cooldown_modifier = 0.5
	end
	if options.equipment_grid then
		spider.equipment_grid = options.equipment_grid
	end
	local consumption = "250kW"
	if not options.have_eff then
		consumption = "750kW"
	end
	if options.have_fusion then
		spider.energy_source = {type = 'void'}
		spider.movement_energy_consumption = consumption
	else
    spider.burner = {
      fuel_category = "chemical",
      effectivity = 1,
      fuel_inventory_size = 5,
      smoke = {
        {
          name = "tank-smoke",
          deviation = {0.25, 0.25},
          frequency = 50,
          position = {0, 1.5},
          starting_frame = 0,
          starting_frame_deviation = 60
        }
      }
    }
    spider.consumption = consumption
	end
	return spider
end


local function make_spider_recipe(options)
	local ingr = {
    {"radar", 2},
    {"raw-fish", 1},
	}
	local ent_ops = {}
	local name = {}
	local function add_ingr(suff, part)
		name[#name+1] = suff
		ingr[#ingr+1] = part
	end
	if options.have_exo then
		add_ingr('exo', {"exoskeleton-equipment", 4})
		ent_ops.leg = 'exo'
	else
		add_ingr('nex', {"steel-plate", 10})
		ent_ops.leg = 'nex'
	end
	ent_ops.have_fusion = options.have_fusion
	if options.have_fusion then
		add_ingr('fus', {"fusion-reactor-equipment", 2})
	else
		add_ingr('nfu', {"engine-unit", 10})
	end
	ent_ops.have_rocket = options.have_rocket
	if options.have_rocket then
		add_ingr('roc', {"rocket-launcher", 4})
	else
		add_ingr('pis', {"pistol", 1})
	end
	if options.have_rcu then
		ent_ops.explore_radius = 5
		add_ingr('rcu', {"rocket-control-unit", 16})
	else
		ent_ops.explore_radius = 1
		add_ingr('nrc', {"electronic-circuit", 20})
	end
	if options.have_lds then
		ent_ops.inventory_size = 80
		ent_ops.health = 1e
		ent_ops.equipment_grid = 'spidertron-equipment-grid'
		add_ingr('lds', {"low-density-structure", 150})
	else
		ent_ops.inventory_size = 40
		ent_ops.health = .5
		ent_ops.equipment_grid = 'small-equipment-grid'
		add_ingr('cop', {'copper-plate', 20})
	end
	ent_ops.have_eff = options.have_eff
	if options.have_eff then
		add_ingr('eff', {"effectivity-module-3", 2})
	else
		add_ingr('nof', {"iron-gear-wheel", 20})
	end
	local full_name = 'spidertron-'..table.concat(name, '-')
  local recipe = {
    type = "recipe",
    name = full_name,
    normal = {
      enabled = true,
      energy_required = 10,
      ingredients = ingr,
      result = full_name,
    },
  }
	local ent = make_spider(ent_ops)
	return recipe, ent
end
